@@ -1,14 +1,15 @@
 import dataclasses
 from datetime import datetime
 import dateutil.parser
-from typing import Generic, TypeVar, Union
+import typing
+from typing import TypeVar, Union
 from enum import EnumMeta
 
 
 T = TypeVar('T')
 
 
-def from_dict(d: dict, t: Generic[T], ignore_extras: bool=True) -> T:
+def from_dict(d: dict, t: T, ignore_extras: bool = True) -> T:
     """
     Initialise an instance of the dataclass t using the values in the dict d
 
@@ -73,9 +74,9 @@ def _convert_to(obj, t, ignore_extras=True):
                 )
         return t(**kwargs)
 
-    elif hasattr(t, '__origin__'):  # i.e List from typing
-        args = t.__args__
-        real_type = t.__origin__
+    elif typing.get_origin(t):  # A typing "mask" type, i.e List/Dict
+        args = typing.get_args(t)
+        real_type = typing.get_origin(t)
 
         # Handle Union types by assuming Optional
         # TODO: support real Union types
@@ -85,6 +86,11 @@ def _convert_to(obj, t, ignore_extras=True):
             else:
                 return _convert_to(obj, args[0])
 
+        if real_type == typing.Literal:
+            if obj not in args:
+                raise TypeError(f'Invalid value "{obj}". Must be one of: {", ".join(args)}')
+            return obj
+
         # validate
         if not isinstance(obj, real_type):
             raise TypeError(f'Object "{obj}" not of expected type {real_type}')
@@ -96,7 +102,8 @@ def _convert_to(obj, t, ignore_extras=True):
             ]
         elif real_type == dict:
             return {
-                _convert_to(k, args[0], ignore_extras=ignore_extras): _convert_to(v, args[1], ignore_extras=ignore_extras)
+                _convert_to(k, args[0], ignore_extras=ignore_extras):
+                    _convert_to(v, args[1], ignore_extras=ignore_extras)
                 for k, v in obj.items()
             }
         else:
@@ -104,6 +111,22 @@ def _convert_to(obj, t, ignore_extras=True):
                 'Type {real_type} currently not supported by howard. '
                 'Consider making a PR.'
             )
+    elif isinstance(t, typing._TypedDictMeta):
+        # is a TypedDict
+        result = {}
+        for key, value in typing.get_type_hints(t).items():
+            if key not in obj:
+                if t.__total__:
+                    raise TypeError(f'Object "{obj}" is missing required key: {key}')
+            else:
+                result[key] = _convert_to(obj[key], value, ignore_extras=ignore_extras)
+        hints = typing.get_type_hints(t)
+        if not ignore_extras:
+            for key in obj:
+                if key not in hints:
+                    raise TypeError(f'Found unexpected key {key} when converting to {t}')
+        return result
+
     elif isinstance(t, EnumMeta):
         return t(obj)
 
